﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;

namespace ELTE.Net.GameServer.Service
{
    /// <summary>
    /// Játékszerver szolgáltatás típusa.
    /// </summary>
    public partial class GameServerService : ServiceBase
    {
        private Boolean _LoggingEnabled; // engedélyezett-e az eseménynaplózás

        /// <summary>
        /// Játékszerver szolgáltatás példányosítása.
        /// </summary>
        public GameServerService()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args) // indítás
        {
            _LoggingEnabled = false;
            try
            {

                if (!EventLog.SourceExists("GameServer")) // ha még nem létezik az eseménynapló
                {
                    EventLog.CreateEventSource("GameServer", "GameServerDiagnosticsLog"); // létrehozzuk
                    _LoggingEnabled = true;
                }
            }
            catch { } // nincs jogosultság

            EventLog eventLog = new System.Diagnostics.EventLog();
            if (_LoggingEnabled)
            {
                eventLog.Source = "GameServer";
                eventLog.Log = "GameServerDiagnosticsLog";
                eventLog.WriteEntry("GameServer indítása folyamatban...", EventLogEntryType.Information);
                // írás az eseménynaplóba
            }

            ServerManager.Instance.Start(); // szerver indítása

            if (_LoggingEnabled)
            {
                if (ServerManager.Instance.Running) // ha sikerült elindítani
                {
                    eventLog.WriteEntry("Szerver elindítva.", EventLogEntryType.Information);
                }
                else
                {
                    eventLog.WriteEntry("Szerver indítása siekrtelen.", EventLogEntryType.Error);
                }
            }
        }

        protected override void OnStop() // leállítás
        {
            ServerManager.Instance.Stop(); // szerver leállítása
            if (_LoggingEnabled)
            {
                EventLog eventLog = new System.Diagnostics.EventLog();
                eventLog.Source = "GameServer";
                eventLog.Log = "GameServerDiagnosticsLog";
                eventLog.WriteEntry("Szerver leállt.", EventLogEntryType.Error);
            }
        }
    }
}
