﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WCFSample.Client.SampleServiceReference;

namespace WCFSample.Client
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// A kliens hálózati kommunikációért felelős modellje.
        /// </summary>
        private SampleServiceClient _client;

        /// <summary>
        /// Kontextus a hálózati kommunikációért felelős kliens modellhez.
        /// </summary>
        private InstanceContext _callbackContext;

        /// <summary>
        /// A kliens feliratkozott-e az értesítésekre.
        /// </summary>
        private Boolean _isSubscribed = false;

        /// <summary>
        /// Konstruktor.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            SampleServiceCallback callbackHandler = new SampleServiceCallback();
            callbackHandler.ItemAdded += new ItemAdeddEventHandler(AddItem);

            _callbackContext = new InstanceContext(callbackHandler);
            _client = new SampleServiceClient(_callbackContext);

            Refresh();
        }

        /// <summary>
        /// Elem rögzítése.
        /// </summary>
        /// <param name="sender">Az eseményt küldő objektum.</param>
        /// <param name="e">Az esemény argumentumai.</param>
        private void Submit(Object sender, RoutedEventArgs e)
        {
            String name = tbName.Text;
            Int32 price;
            try
            {
                price = Int32.Parse(tbPrice.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("A megadott ár nem egész szám!", "Hibás érték", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            _client.AddItem(new Item { Name = name, Price = price });

            // Ha ez a kliens nem iratkozott fel az értesítésekre, akkor lekérjük az összes elemet a megjelenítés frissítéséhez.
            if (!_isSubscribed)
                Refresh(sender, e);

            tbName.Text = String.Empty;
            tbPrice.Text = String.Empty;
        }

        /// <summary>
        /// Elemek frissítése.
        /// </summary>
        private void Refresh()
        {
            Item[] items = _client.GetItems();
            lbxItems.Items.Clear();
            foreach (Item item in items)
                lbxItems.Items.Add(String.Format("Név: {0}, Ár: {1}", item.Name, item.Price));
        }

        /// <summary>
        /// Elemek frissítése.
        /// </summary>
        /// <param name="sender">Az eseményt küldő objektum.</param>
        /// <param name="e">Az esemény argumentumai.</param>
        private void Refresh(Object sender, RoutedEventArgs e)
        {
            Refresh();
        }

        /// <summary>
        /// Új elem felvétele.
        /// </summary>
        /// <param name="sender">Az eseményt küldő objektum.</param>
        /// <param name="item">Az új elem.</param>
        private void AddItem(Object sender, Item item)
        {
            if (Dispatcher.Thread != Thread.CurrentThread)
            {
                Dispatcher.BeginInvoke(new Action(
                    () => AddItem(sender, item)));
                return;
            }

            lbxItems.Items.Add(String.Format("Név: {0}, Ár: {1}", item.Name, item.Price));
        }

        /// <summary>
        /// Feliratkozás az értesítésekre.
        /// </summary>
        /// <param name="sender">Az eseményt küldő objektum.</param>
        /// <param name="e">Az esemény argumentumai.</param>
        private void Subscribe(Object sender, RoutedEventArgs e)
        {
            _client.Subscribe();
            _isSubscribed = true;
            btnSubscribe.IsEnabled = false;
            btnUnsubscribe.IsEnabled = true;
        }

        /// <summary>
        /// Leiratkozás az értesítésekről.
        /// </summary>
        /// <param name="sender">Az eseményt küldő objektum.</param>
        /// <param name="e">Az esemény argumentumai.</param>
        private void Unsubscribe(Object sender, RoutedEventArgs e)
        {
            _client.Unsubscribe();
            _isSubscribed = false;
            btnUnsubscribe.IsEnabled = false;
            btnSubscribe.IsEnabled = true;            
        }
    }
}
